/*
 * NumberComparatorTest.java
 * Part of the B'am library - a collection of general-purpose utility classes.
 * 
 * Copyright (C)2006 Stephen Battey
 * Contact: resource@lucidviews.net
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

package net.lucidviews.util;

import junit.framework.TestCase;


/**
 * Test harness for the <code>NumberComparator</code> class.
 * <br />
 * <br />
 * @author Stephen Battey
 * @version $Revision: 1.1 $
 * @since 1.0
 */
public class NumberComparatorTest extends TestCase
{
	
	/*
	 * Static methods
	 */
	
	/**
	 * Runs the test in the command console.
	 * <br />
	 * @param args not used
	 */
	public static void main( String[] args )
	{
		junit.textui.TestRunner.run( NumberComparatorTest.class );
	}
	
	
	/*
	 * Overridden methods of TestCase base class
	 */
	
	public void setUp() throws Exception
	{
		super.setUp();
	}
	
	
	/*
	 * Test cases
	 */
	
	public void testInts()
	{
		Integer v1 = new Integer( -4574389 );
		Integer v2 = new Integer( -4574389 );
		Integer v3 = new Integer( 3 );
		Integer v4 = new Integer( 349743895 );
		
		assertTrue( NumberComparator.ASCENDING.compare( v1, v2 ) == 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v1, v2 ) == 0 );
		
		assertTrue( NumberComparator.ASCENDING.compare( v1, v3 ) < 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v1, v3 ) > 0 );
		
		assertTrue( NumberComparator.ASCENDING.compare( v3, v1 ) > 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v3, v1 ) < 0 );
		
		assertTrue( NumberComparator.ASCENDING.compare( v3, v4 ) < 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v3, v4 ) > 0 );
		
		assertTrue( NumberComparator.ASCENDING.compare( v4, v3 ) > 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v4, v3 ) < 0 );
	}
	
	public void testLongs()
	{
		Long v1 = new Long( -45743895637l );
		Long v2 = new Long( -45743895637l );
		Long v3 = new Long( 3 );
		Long v4 = new Long( 3497438953489l );
		
		assertTrue( NumberComparator.ASCENDING.compare( v1, v2 ) == 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v1, v2 ) == 0 );
		
		assertTrue( NumberComparator.ASCENDING.compare( v1, v3 ) < 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v1, v3 ) > 0 );
		
		assertTrue( NumberComparator.ASCENDING.compare( v3, v1 ) > 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v3, v1 ) < 0 );
		
		assertTrue( NumberComparator.ASCENDING.compare( v3, v4 ) < 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v3, v4 ) > 0 );
		
		assertTrue( NumberComparator.ASCENDING.compare( v4, v3 ) > 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v4, v3 ) < 0 );
	}
	
	public void testFloats()
	{
		Float v1 = new Float( -4574389.4589345 );
		Float v2 = new Float( -4574389.4589345 );
		Float v3 = new Float( 2.69 );
		Float v4 = new Float( 349743895.3489 );
		
		assertTrue( NumberComparator.ASCENDING.compare( v1, v2 ) == 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v1, v2 ) == 0 );
		
		assertTrue( NumberComparator.ASCENDING.compare( v1, v3 ) < 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v1, v3 ) > 0 );
		
		assertTrue( NumberComparator.ASCENDING.compare( v3, v1 ) > 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v3, v1 ) < 0 );
		
		assertTrue( NumberComparator.ASCENDING.compare( v3, v4 ) < 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v3, v4 ) > 0 );
		
		assertTrue( NumberComparator.ASCENDING.compare( v4, v3 ) > 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v4, v3 ) < 0 );
	}
	
	public void testDoubles()
	{
		Double v1 = new Double( -45743894589345.45873497 );
		Double v2 = new Double( -45743894589345.45873497 );
		Double v3 = new Double( 2.69 );
		Double v4 = new Double( 3.49743895348934 );
		
		assertTrue( NumberComparator.ASCENDING.compare( v1, v2 ) == 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v1, v2 ) == 0 );
		
		assertTrue( NumberComparator.ASCENDING.compare( v1, v3 ) < 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v1, v3 ) > 0 );
		
		assertTrue( NumberComparator.ASCENDING.compare( v3, v1 ) > 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v3, v1 ) < 0 );
		
		assertTrue( NumberComparator.ASCENDING.compare( v3, v4 ) < 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v3, v4 ) > 0 );
		
		assertTrue( NumberComparator.ASCENDING.compare( v4, v3 ) > 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v4, v3 ) < 0 );
	}
	
	public void testIntAndLong()
	{
		Long v1 = new Long( -45743895637l );
		Integer v3 = new Integer( 3 );
		Long v4 = new Long( 3497438953489l );
		
		assertTrue( NumberComparator.ASCENDING.compare( v1, v3 ) < 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v1, v3 ) > 0 );
		
		assertTrue( NumberComparator.ASCENDING.compare( v3, v1 ) > 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v3, v1 ) < 0 );
		
		assertTrue( NumberComparator.ASCENDING.compare( v3, v4 ) < 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v3, v4 ) > 0 );
		
		assertTrue( NumberComparator.ASCENDING.compare( v4, v3 ) > 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v4, v3 ) < 0 );
	}
	
	public void testFloatAndDouble()
	{
		Double v1 = new Double( -45743894589345.45873497 );
		Float v3 = new Float( 2.69 );
		Double v4 = new Double( 3.49743895348934 );
		
		assertTrue( NumberComparator.ASCENDING.compare( v1, v3 ) < 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v1, v3 ) > 0 );
		
		assertTrue( NumberComparator.ASCENDING.compare( v3, v1 ) > 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v3, v1 ) < 0 );
		
		assertTrue( NumberComparator.ASCENDING.compare( v3, v4 ) < 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v3, v4 ) > 0 );
		
		assertTrue( NumberComparator.ASCENDING.compare( v4, v3 ) > 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v4, v3 ) < 0 );
	}
	
	public void testLongAndDouble()
	{
		// this test ensures we don't loose precision casting a long to a double
		Long v1 = new Long( Long.MIN_VALUE );
		Double v3 = new Double( 3.0 );
		Long v4 = new Long( Long.MAX_VALUE );
		
		assertTrue( NumberComparator.ASCENDING.compare( v1, v3 ) < 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v1, v3 ) > 0 );
		
		assertTrue( NumberComparator.ASCENDING.compare( v3, v1 ) > 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v3, v1 ) < 0 );
		
		assertTrue( NumberComparator.ASCENDING.compare( v3, v4 ) < 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v3, v4 ) > 0 );
		
		assertTrue( NumberComparator.ASCENDING.compare( v4, v3 ) > 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v4, v3 ) < 0 );
	}
	
	public void testLongAndFloat()
	{
		// again try min and max long values
		// (although the comparison should still be performed using doubles)
		Long v1 = new Long( Long.MIN_VALUE );
		Float v3 = new Float( 3.0 );
		Long v4 = new Long( Long.MAX_VALUE );
		
		assertTrue( NumberComparator.ASCENDING.compare( v1, v3 ) < 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v1, v3 ) > 0 );
		
		assertTrue( NumberComparator.ASCENDING.compare( v3, v1 ) > 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v3, v1 ) < 0 );
		
		assertTrue( NumberComparator.ASCENDING.compare( v3, v4 ) < 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v3, v4 ) > 0 );
		
		assertTrue( NumberComparator.ASCENDING.compare( v4, v3 ) > 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v4, v3 ) < 0 );
	}
	
	public void testIntAndFloat()
	{
		Integer v1 = new Integer( Integer.MIN_VALUE );
		Float v3 = new Float( 3.0 );
		Integer v4 = new Integer( Integer.MAX_VALUE );
		
		assertTrue( NumberComparator.ASCENDING.compare( v1, v3 ) < 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v1, v3 ) > 0 );
		
		assertTrue( NumberComparator.ASCENDING.compare( v3, v1 ) > 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v3, v1 ) < 0 );
		
		assertTrue( NumberComparator.ASCENDING.compare( v3, v4 ) < 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v3, v4 ) > 0 );
		
		assertTrue( NumberComparator.ASCENDING.compare( v4, v3 ) > 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v4, v3 ) < 0 );
	}
	
	public void testIntAndDouble()
	{
		Integer v1 = new Integer( Integer.MIN_VALUE );
		Double v3 = new Double( 3.0 );
		Integer v4 = new Integer( Integer.MAX_VALUE );
		
		assertTrue( NumberComparator.ASCENDING.compare( v1, v3 ) < 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v1, v3 ) > 0 );
		
		assertTrue( NumberComparator.ASCENDING.compare( v3, v1 ) > 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v3, v1 ) < 0 );
		
		assertTrue( NumberComparator.ASCENDING.compare( v3, v4 ) < 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v3, v4 ) > 0 );
		
		assertTrue( NumberComparator.ASCENDING.compare( v4, v3 ) > 0 );
		assertTrue( NumberComparator.DESCENDING.compare( v4, v3 ) < 0 );
	}
	
	public void testEquals()
	{
		assertTrue( NumberComparator.ASCENDING.equals( NumberComparator.ASCENDING ) );
		assertTrue( NumberComparator.DESCENDING.equals( NumberComparator.DESCENDING ) );
		assertFalse( NumberComparator.DESCENDING.equals( NumberComparator.ASCENDING ) );
		assertFalse( NumberComparator.ASCENDING.equals( NumberComparator.DESCENDING ) );
	}
	
}


/*
 * Change History
 *
 * $Log: NumberComparatorTest.java,v $
 * Revision 1.1  2006/09/13 20:09:05  Steve
 * Initial revision.
 *
 */